@@ -61,7 +61,7 @@
 #define MINVERSION 0 /* API compatible, ABI may change, functional
 		      * enhancements only, consumer can be left unchanged if
 		      * enhancements are not considered */
-#define PATCHLEVEL 4 /* API / ABI compatible, no functional changes, no
+#define PATCHLEVEL 5 /* API / ABI compatible, no functional changes, no
 		      * enhancements, bug fixes only */
 
 static int Verbosity = 0;
@@ -98,7 +98,7 @@ static int Entropy_avail_fd = 0;
 
 #define ENTROPYBYTES 32
 #define OVERSAMPLINGFACTOR 2
-#define ENTROPYTRHESH 1024
+#define ENTROPYTHRESH 1024
 #define ENTROPYAVAIL "/proc/sys/kernel/random/entropy_avail"
 #define LRNG_FILE "/proc/sys/kernel/random/lrng_type"
 
@@ -118,38 +118,66 @@ static void usage(void)
 	char version[30];
 
 	memset(version, 0, 30);
-	jentrng_versionstring(version, 30);
+	jentrng_versionstring(version, sizeof(version));
 
 	fprintf(stderr, "\njitterentropy rngd feeding entropy to input_pool of Linux RNG\n");
 	fprintf(stderr, "Version %s\n\n", version);
 	fprintf(stderr, "Reported numeric version number of jent library %u\n\n", ver);
 	fprintf(stderr, "Usage:\n");
-	fprintf(stderr, "\t-v\tVerbose logging, multiple options increase verbosity\n");
-	fprintf(stderr, "\t\tVerbose logging implies running in foreground\n");
-	fprintf(stderr, "\t-p\tWrite daemon PID to file\n");
+	fprintf(stderr, "\t-h --help\tThis help information\n");
+	fprintf(stderr, "\t   --version\tPrint version\n");
+	fprintf(stderr, "\t-v --verbose\tVerbose logging, multiple options increase verbosity\n");
+	fprintf(stderr, "\t\t\tVerbose logging implies running in foreground\n");
+	fprintf(stderr, "\t-p --pid\tWrite daemon PID to file\n");
 	exit(1);
 }
 
 static void parse_opts(int argc, char *argv[])
 {
 	int c = 0;
+	char version[30];
+
 	while (1) {
 		int opt_index = 0;
 		static struct option opts[] = {
 			{"verbose", 0, 0, 0},
 			{"pid", 1, 0, 0},
+			{"help", 0, 0, 0},
+			{"version", 0, 0, 0},
 			{0, 0, 0, 0}
 		};
-		c = getopt_long(argc, argv, "vp:", opts, &opt_index);
+		c = getopt_long(argc, argv, "vph:", opts, &opt_index);
 		if (-1 == c)
 			break;
 		switch (c) {
+		case 0:
+			switch (opt_index) {
+			case 0:
+				Verbosity++;
+				break;
+			case 1:
+				Pidfile = optarg;
+				break;
+			case 2:
+				usage();
+				break;
+			case 3:
+				jentrng_versionstring(version, sizeof(version));
+				fprintf(stderr, "Version %s\n", version);
+				break;
+			default:
+				usage();
+			}
+			break;
 		case 'v':
 			Verbosity++;
 			break;
 		case 'p':
 			Pidfile = optarg;
 			break;
+		case 'h':
+			usage();
+			break;
 		default:
 			usage();
 		}
@@ -222,7 +250,7 @@ static size_t write_random(struct kernel_rng *rng, char *buf, size_t len,
 	if (0 > ret)
 		dolog(LOG_WARN, "Error injecting entropy: %s", strerror(errno));
 	else {
-		dolog(LOG_DEBUG, "Injected %u bytes with an entropy contnt of %u bytes of entropy",
+		dolog(LOG_DEBUG, "Injected %u bytes with an entropy count of %u bytes of entropy",
 		      len, entropy_bytes);
 		written = len;
 	}
@@ -247,9 +275,11 @@ static size_t gather_entropy(struct kernel_rng *rng)
 	if (sizeof(buf) != ret)
 		dolog(LOG_WARN, "Injected %lu bytes into %s, expected %d",
 			ret, rng->dev, sizeof(buf));
+	else
+		ret = sizeof(buf);
 	memset_secure(buf, 0, sizeof(buf));
 
-	return sizeof(buf);
+	return ret;
 }
 
 static int read_entropy_avail(int fd)
@@ -299,7 +329,7 @@ static void sig_entropy_avail(int sig)
 
 	if (0 == entropy)
 		goto out;
-	if (ENTROPYTRHESH < entropy) {
+	if (ENTROPYTHRESH < entropy) {
 		dolog(LOG_DEBUG, "Sufficient entropy %d available", entropy);
 		goto out;
 	}
@@ -527,9 +557,9 @@ int main(int argc, char *argv[])
 		dolog(LOG_ERR, "Program must start as root!");
 
 	parse_opts(argc, argv);
+	alloc();
 	if (0 == Verbosity)
 		daemonize();
-	alloc();
 	install_term();
 	install_alarm();
 	select_fd();
